#! /usr/bin/env python

import sys, os
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import scipy.constants as const
sys.path.append("../../BART/code/")
import reader as rd
import constants as c
import bestFit as bf
import makeatm as ma
import PT as pt
sys.path.append("../../BART/modules/MCcubed/MCcubed/plots/")
import mcplots as mcp


plt.ion()


def readatm(atmfile):
    # Open the atm file and read it
    atmfoo = open(atmfile, 'r')
    lines  = atmfoo.readlines()

    # Store the molecule info
    mols = lines[9].split()

    # Find where the layer data starts
    for i in range(len(lines)):
        line = lines[i].split()
        try:
            if line[0] == '#Radius':
                break
        except:
            continue

    # Trim atm file
    lines = lines[i + 1:]

    # Array to hold atm file info
    atminfo = np.zeros((len(lines), len(mols) + 3), dtype=float)
    # Note that +3 is to hold radius, pressure, and temperature

    # Read in data
    for i in range(len(lines)):
        # Split info for layer `i`
        line = lines[i].split()
        for j in range(3 + len(mols)): # rad, press, temp, mol1, mol2, ...
            # Read in each column
            atminfo[i, j] = float(line[j])

    # Make sure atmosphere is ordered from bottom to top
    if atminfo[0, 0] > atminfo[1, 0]:
        for i in range(atminfo.shape[1]):
            atminfo[:, i] = atminfo[:, i][::-1]

    return mols, atminfo


def retrievedPT(datadir, atmfile, tepfile, nmol, solution, 
                outname, outdir=None, T_int=100.):
    """
    Inputs
    ------
    datadir:  string. Path/to/directory containing BART-formatted output.
    atmfile:  string. Path/to/atmospheric model file.
    tepfile:  string. Path/to/Transiting ExoPlanet file.
    nmol:     int.    Number of molecules being fit by MCMC.
    solution: string. Geometry of the system. 'eclipse' or 'transit'. 
    outname:  string. File name of resulting plot.
    outdir:   string. Path/to/dir to save `outname`. If None, defaults 
                      to the results directory of BARTTest if `datadir` is 
                      within BARTTest.
    T_int:    float.  Internal planetary temperature. Default is 100 K.
    """
    # Set outdir if not specified
    if outdir is None:
        try:
            if datadir[-1] != '/':
                datadir = datadir + '/'
            outdir = 'results'.join(datadir.rsplit('code-output',            \
                                                   1)).rsplit('/', 2)[0] + '/'
            if not os.path.isdir(outdir):
                os.makedirs(outdir)
        except:
            print("Data directory not located within BARTTest.")
            print("Please specify an output directory `outdir` and try again.")
            sys.exit(1)

    # Read g_surf and R_planet from TEP file
    grav, Rp = ma.get_g(tepfile)

    # Read star data from TEP file, and semi-major axis
    R_star, T_star, sma, gstar = bf.get_starData(tepfile)

    # Read atmfile
    mols, atminfo = readatm(atmfile)
    pressure = atminfo[:, 1]

    # Read MCMC output file
    MCfile = datadir + 'MCMC.log'
    bestP, uncer = bf.read_MCMC_out(MCfile)
    allParams = bestP
    # Get number of burned iterations
    foo   = open(MCfile, 'r')
    lines = foo.readlines()
    foo.close()
    line = [foop for foop in lines if " Burned in iterations per chain:" in foop]
    burnin = int(line[0].split()[-1])

    # Figure out number of parameters
    nparams   = len(allParams)
    nradfit   = int(solution == 'transit')
    nPTparams = nparams - nmol - nradfit
    PTparams  = allParams[:nPTparams]

    # Plot the best PT profile
    kappa, gamma1, gamma2, alpha, beta = PTparams
    best_T = pt.PT_line(pressure, kappa,  gamma1, gamma2, alpha, beta, 
                        R_star,   T_star, T_int,  sma,    grav*1e2, 'const')

    # Load MCMC data
    MCMCdata = datadir + 'output.npy'
    data     = np.load(MCMCdata)
    nchains, npars, niter = np.shape(data)

    # Make datacube from MCMC data
    data_stack = data[0,:,burnin:]
    for c in np.arange(1, nchains):
        data_stack = np.hstack((data_stack, data[c, :, burnin:]))

    # Datacube of PT profiles
    PTprofiles = np.zeros((np.shape(data_stack)[1], len(pressure)))

    curr_PTparams = PTparams

    for k in np.arange(0, np.shape(data_stack)[1]):
        j = 0
        for i in np.arange(len(PTparams)):
            curr_PTparams[i] = data_stack[j,k]
            j +=1
        kappa, gamma1, gamma2, alpha, beta = curr_PTparams
        PTprofiles[k] = pt.PT_line(pressure, kappa,  gamma1, gamma2, 
                                   alpha, beta, R_star, T_star, T_int, 
                                   sma, grav*1e2, 'const')


    # Get percentiles (for 1, 2-sigma boundaries):
    low1 = np.percentile(PTprofiles, 16.0, axis=0)
    hi1  = np.percentile(PTprofiles, 84.0, axis=0)
    low2 = np.percentile(PTprofiles,  2.5, axis=0)
    hi2  = np.percentile(PTprofiles, 97.5, axis=0)
    median = np.median(PTprofiles, axis=0)

    # Plot and save figure
    plt.figure(2)
    plt.clf()
    ax=plt.subplot(111)
    ax.fill_betweenx(pressure, low2, hi2, facecolor="#62B1FF", edgecolor="0.5")
    ax.fill_betweenx(pressure, low1, hi1, facecolor="#1873CC",
                                                           edgecolor="#1873CC")
    plt.semilogy(median, pressure, "-", lw=2, label='Median',color="k")
    plt.semilogy(best_T, pressure, "-", lw=2, label="Best fit", color="r")
    plt.semilogy(atminfo[:,2], pressure, "--", lw=2, label='Input', color='r')
    plt.ylim(pressure[0], pressure[-1])
    plt.legend(loc="best")
    plt.xlabel("Temperature  (K)", size=15)
    plt.ylabel("Pressure  (bar)",  size=15)
    plt.savefig(outdir + outname)
    plt.close()


def retrievedabun(datadir, outname, atminput, 
                  atmretrv='bestFit.atm', outdir=None):
    """
    Inputs
    ------
    datadir:  string. Path/to/directory containing BART-formatted output.
    outname:  string. File name of resulting plot. Do not include path, use 
                      `outdir` for that purpose.
    atminput:  string. Path/to/atmospheric model file used for input.
    atmretrv:  string. Name of atmospheric model file retrieved with respect 
                       to `datadir`. Default is 'bestFit.atm', the default 
                       name of BART's output.
    outdir:   string. Path/to/dir to save `outname`. If None, defaults 
                      to the results directory of BARTTest.
    """
    # Make sure datadir has a trailing /
    if datadir[-1] != '/':
        datadir = datadir + '/'
    # Set outdir if not specified. If it is, ensure trailing /
    if outdir is None:
        try:
            outdir = 'results'.join(datadir.rsplit('code-output',            \
                                                   1)).rsplit('/', 2)[0] + '/'
        except:
            print("Data directory not located within BARTTest.")
            print("Please specify an output directory `outdir` and try again.")
            sys.exit(1)
    elif outdir[-1] != '/':
        outdir = outdir + '/'
    # Ensure outdir exists
    if not os.path.isdir(outdir):
        os.makedirs(outdir)

    # Read the atm files
    molin, atmin = readatm(atminput)
    molrs, atmrs = readatm(datadir+atmretrv)

    # Only worry about molecules that are being fit
    ignore = {'H', 'C', 'N', 'O', 'He', 'H2', 'N2'}

    # Colors for plotting
    cols = ['deep pink', 'medium purple', 'indianred', 'teal', 'gray', \
            'green', 'blue', 'red', 'purple', 'lime', 'black', 'maroon', 'aqua']

    # Plot abundances
    for i in range(len(molrs)):
        if molrs[i] in ignore:
            continue
        try:
            ind = molin.index(molrs[i])
        except:
            continue
        plt.plot(atmin[:,ind+3], atmin[:,1], ls='--', lw=2, \
                 color=cols[i], label='Input '+molin[ind])
        plt.plot(atmrs[:,i  +3], atmrs[:,1], ls='-', lw=2, color=cols[i], \
                 label='Retrieved '+molrs[i])

    # Labels, legend, save plot
    plt.xlabel('Molar Mixing Fraction')
    plt.xscale('log')
    plt.ylabel('Pressure [bars]')
    plt.yscale('log')
    plt.gca().invert_yaxis()
    plt.legend(loc='lower left', prop={'size': 9})
    plt.savefig(outdir+outname)
    plt.close()


def retrievedpost(datadir, outname, pnames, true, shift, 
                  fpost='output.npy', outdir=None, pltlgd=False):
    """
    datadir : string. Path/to/directory containing BART-formatted output.
    outname : string. File name of resulting plot. Do not include path, use 
                      `outdir` for that purpose.
    pnames  : list, strings. Parameter names.
    true    : array.  True values.
    shift   : array.  Shift values of the posterior 
                      (e.g., to correct for the assumed starting abundance)
    fpost   : string. File name of .NPY file containing the posterior.
    outdir  : string. Path/to/dir to save `outname`. If None, defaults 
                      to the results directory of BARTTest.
    """
    # Make sure datadir has a trailing /
    if datadir[-1] != '/':
        datadir = datadir + '/'
    # Set outdir if not specified. If it is, ensure trailing /
    if outdir is None:
        try:
            outdir = 'results'.join(datadir.rsplit('code-output',            \
                                                   1)).rsplit('/', 2)[0] + '/'
        except:
            print("Data directory not located within BARTTest.")
            print("Please specify an output directory `outdir` and try again.")
            sys.exit(1)
    elif outdir[-1] != '/':
        outdir = outdir + '/'

    # Read MCMC output file
    MCfile = datadir + 'MCMC.log'
    bestP, uncer = bf.read_MCMC_out(MCfile)
    allParams = bestP
    # Get number of burned iterations
    foo   = open(MCfile, 'r')
    lines = foo.readlines()
    foo.close()
    line = [foop for foop in lines if " Burned in iterations per chain:" in foop]
    burnin = int(line[0].split()[-1])

    # Load the data
    data = np.load(datadir+fpost)
    data_stack = data[0,:,burnin:]
    for c in np.arange(1, data.shape[0]):
        data_stack = np.hstack((data_stack, data[c, :, burnin:]))
    # Remove T(p) profiles
    data_stack = data_stack[5:]
    # Shift
    data_stack += shift[:,None]

    if data_stack.shape[0] != len(pnames):
        raise ValueError("Posterior dimensionality does not match number of parameter names.")

    # Plot it
    histogram(data_stack, parname=pnames, truepars=true, savefile=outdir+outname)


def histogram(stack, parname=None, truepars=None, 
              fignum=-12, fs=30, savefile=None, bins=60, pltlgd=False):
    """
    Plots the probability density functions of 1D marginalized posteriors 
    for an MCMC runs.
    
    Inputs
    ------
    stack   : array.  Posterior, shaped (nparams, niterations)
    parname : list, strings. Parameter names.
    truepars: array.  True values for each parameter considered.
    fs      : int.    Font size for plots.
    savefile: string. Path/to/file where the plot will be saved.
    bins    : int.    Number of bins for the histograms.
    """
    npars, niter = np.shape(stack)

    # Set default parameter names:
    if parname is None:
        namelen = int(2+np.log10(np.amax([npars-1,1])))
        parname = np.zeros(npars, "<U%d"%namelen)
        for i in np.arange(npars):
            parname[i] = "P" + str(i).zfill(namelen-1)

    # Set number of rows:
    if npars < 10:
        nperrow = 3
    else:
        nperrow = 4
    nrows = (npars - 1)//nperrow + 1
    # Set number of columns:
    if   npars > 9:
        ncolumns = 4
    elif npars > 4:
        ncolumns = 3
    else:
        ncolumns = (npars+2)//3 + (npars+2)%3  # (Trust me!)

    histheight = 4 + 4*(nrows)
    if nrows == 1:
        bottom = 0.25
    else:
        bottom = 0.15

    fig = plt.figure(fignum, figsize=(18, histheight))
    plt.clf()
    plt.subplots_adjust(left=0.1, right=0.95, bottom=bottom, top=0.9,
                        hspace=0.8, wspace=0.25)

    for i in np.arange(npars):
        ax = plt.subplot(nrows, ncolumns, i+1)
        a  = plt.xticks(size=fs-4, rotation=90)
        a  = plt.yticks(size=fs-4)
        if i%ncolumns == 0:
            plt.ylabel('Normalized PDF', size=fs)
        plt.xlabel(parname[i], size=fs)
        rng   = stack[i].min(), stack[i].max()
        a = plt.hist(stack[i], bins, range=rng, density=True, color='b')
        # Add true value if given
        if truepars is not None:
            ax.axvline(truepars[i], color='red', lw=4, zorder=20)
    fig.align_labels()
    if pltlgd and truepars is not None:
        hndls = [] + [mpl.lines.Line2D([], [], color='red', lw=4, label='True value')]
        plt.legend(handles=hndls, prop={'size':fs/1.5}, loc='upper left', 
                   bbox_to_anchor=(1, 0.8))

    if savefile is not None:
        plt.savefig(savefile, bbox_inches='tight')
    plt.close()
    

if __name__ == '__main__':
    # Common inputs
    datadirbase = '../code-output/01BART/'
    testdirbase = '../tests/'
    tepfile     = '../tests/00inputs/HD189733b.tep'
    outdir      = '../results/01BART/'
    fext        = '.pdf'

    # Make the plots

    # Posteriors
    print("Making posterior plots w/ true values...\n")
    pnames_ecl = ['H$_2$O', 'CO$_2$', 'CO', 'CH$_4$', 'NH$_3$']
    pnames_tra = ['R$_P$', 'H$_2$O', 'CO$_2$', 'CO', 'CH$_4$', 'NH$_3$']

    try:
        print('  s02hjclearnoinv...\n')
        retrievedpost(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr50_hires/', 
                      's02hjclearnoinv-ecl-uni-noiseless-snr50-grid1.0-post'+fext, pnames_ecl, 
                      true=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      shift=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedpost(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr50_hires_finegrid/', 
                      's02hjclearnoinv-ecl-uni-noiseless-snr50-grid0.5-post'+fext, pnames_ecl, 
                      true=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      shift=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedpost(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr50_hires_finergrid/', 
                      's02hjclearnoinv-ecl-uni-noiseless-snr50-grid0.25-post'+fext, pnames_ecl, 
                      true=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      shift=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedpost(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr50_hires_finestgrid/', 
                      's02hjclearnoinv-ecl-uni-noiseless-snr50-grid0.1-post'+fext, pnames_ecl, 
                      true=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      shift=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      outdir=outdir, pltlgd=True)
    except Exception as e:
        print(e)
        pass

    """
    try:
        print('  s02hjclearnoinv...\n')
        retrievedPT(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr50_hires_finestgrid/', 
                    testdirbase+'c02hjclearnoinv/noinv_uni.atm', 
                    tepfile, 5, 'eclipse', 's02hjclearnoinv-ecl-uni-noiseless-snr50-grid0.1-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedpost(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr25/', 
                      's02hjclearnoinv-ecl-uni-noiseless-snr25-post.png', pnames_ecl, 
                      true=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      shift=np.array([-6, -6, -6, -6, -9]), 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedpost(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr100/', 
                      's02hjclearnoinv-ecl-uni-noiseless-snr100-post.png', pnames_ecl, 
                      true=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      shift=np.array([-6, -6, -6, -6, -9]), 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedpost(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr6-7/', 
                      's02hjclearnoinv-ecl-uni-noiseless-snr6-post.png', pnames_ecl, 
                      true=np.array([np.log10(5e-5), -5, -4, np.log10(5e-6), -6]), 
                      shift=np.array([-6, -6, -6, -6, -9]), 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedPT(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr25/', 
                    testdirbase+'c02hjclearnoinv/noinv.tea', 
                    tepfile, 5, 'eclipse', 's02hjclearnoinv-ecl-uni-noiseless-snr25-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedPT(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr100/', 
                    testdirbase+'c02hjclearnoinv/noinv.tea', 
                    tepfile, 5, 'eclipse', 's02hjclearnoinv-ecl-uni-noiseless-snr100-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedPT(datadirbase+'s02hjclearnoinv-ecl-uni_noiseless_snr6-7/', 
                    testdirbase+'c02hjclearnoinv/noinv.tea', 
                    tepfile, 5, 'eclipse', 's02hjclearnoinv-ecl-uni-noiseless-snr6-PT.png', outdir)
    except Exception as e:
        print(e)
        pass




    print('Plotting the PT profiles...\n')
    try:
        print('  s01hjcleariso...\n')
        retrievedPT(datadirbase+'s01hjcleariso-ecl/', 
                    testdirbase+'c01hjcleariso/iso.tea', 
                    tepfile, 5, 'eclipse', 's01hjcleariso-ecl-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedPT(datadirbase+'s01hjcleariso-tra/', 
                    testdirbase+'c01hjcleariso/iso.tea', 
                    tepfile, 5, 'transit', 's01hjcleariso-tra-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedPT(datadirbase+'s02hjclearnoinv-ecl/', 
                    testdirbase+'c02hjclearnoinv/noinv.tea', 
                    tepfile, 5, 'eclipse', 's02hjclearnoinv-ecl-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedPT(datadirbase+'s02hjclearnoinv-tra/', 
                    testdirbase+'c02hjclearnoinv/noinv.tea', 
                    tepfile, 5, 'transit', 's02hjclearnoinv-tra-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s03hjclearinv...\n')
        retrievedPT(datadirbase+'s03hjclearinv-ecl/', 
                    testdirbase+'c03hjclearinv/inv.tea', 
                    tepfile, 5, 'eclipse', 's03hjclearinv-ecl-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedPT(datadirbase+'s03hjclearinv-tra/', 
                    testdirbase+'c03hjclearinv/inv.tea', 
                    tepfile, 5, 'transit', 's03hjclearinv-tra-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    # Uniform cases

    try:
        print('  s01hjcleariso...\n')
        retrievedPT(datadirbase+'s01hjcleariso-ecl-uni/', 
                    testdirbase+'c01hjcleariso/iso_uni.atm', 
                    tepfile, 5, 'eclipse', 's01hjcleariso-ecl-uni-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedPT(datadirbase+'s01hjcleariso-tra-uni/', 
                    testdirbase+'c01hjcleariso/iso_uni.atm', 
                    tepfile, 5, 'transit', 's01hjcleariso-tra-uni-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedPT(datadirbase+'s02hjclearnoinv-ecl-uni/', 
                    testdirbase+'c02hjclearnoinv/noinv_uni.atm', 
                    tepfile, 5, 'eclipse', 's02hjclearnoinv-ecl-uni-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedPT(datadirbase+'s02hjclearnoinv-tra-uni/', 
                    testdirbase+'c02hjclearnoinv/noinv_uni.atm', 
                    tepfile, 5, 'transit', 's02hjclearnoinv-tra-uni-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s03hjclearinv...\n')
        retrievedPT(datadirbase+'s03hjclearinv-ecl-uni/', 
                    testdirbase+'c03hjclearinv/inv_uni.atm', 
                    tepfile, 5, 'eclipse', 's03hjclearinv-ecl-uni-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedPT(datadirbase+'s03hjclearinv-tra-uni/', 
                    testdirbase+'c03hjclearinv/inv_uni.atm', 
                    tepfile, 5, 'transit', 's03hjclearinv-tra-uni-PT.png', outdir)
    except Exception as e:
        print(e)
        pass

    print('Plotting the vertical abundance profiles...\n')

    try:
        print('  s01hjcleariso...\n')
        retrievedabun(datadirbase+'s01hjcleariso-ecl/', 
                      's01hjcleariso-ecl-abun.png', 
                      testdirbase+'c01hjcleariso/iso.tea', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s01hjcleariso-tra/', 
                      's01hjcleariso-tra-abun.png', 
                      testdirbase+'c01hjcleariso/iso.tea', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s02hjclearnoinv...\n')
        retrievedabun(datadirbase+'s02hjclearnoinv-ecl/', 
                      's02hjclearnoinv-ecl-abun.png', 
                      testdirbase+'c02hjclearnoinv/noinv.tea', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s02hjclearnoinv-tra/', 
                      's02hjclearnoinv-tra-abun.png', 
                      testdirbase+'c02hjclearnoinv/noinv.tea', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        print('  s03hjclearinv...\n')
        retrievedabun(datadirbase+'s03hjclearinv-ecl/', 
                      's03hjclearinv-ecl-abun.png', 
                      testdirbase+'c03hjclearinv/inv.tea', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s03hjclearinv-tra/', 
                      's03hjclearinv-tra-abun.png', 
                      testdirbase+'c03hjclearinv/inv.tea', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    # Uniform cases
    try:
        retrievedabun(datadirbase+'s01hjcleariso-ecl-uni/', 
                      's01hjcleariso-ecl-uni-abun.png', 
                      testdirbase+'c01hjcleariso/iso_uni.atm', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s01hjcleariso-tra-uni/', 
                      's01hjcleariso-tra-uni-abun.png', 
                      testdirbase+'c01hjcleariso/iso_uni.atm', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s02hjclearnoinv-ecl-uni/', 
                      's02hjclearnoinv-ecl-uni-abun.png', 
                      testdirbase+'c02hjclearnoinv/noinv_uni.atm', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s02hjclearnoinv-tra-uni/', 
                      's02hjclearnoinv-tra-uni-abun.png', 
                      testdirbase+'c02hjclearnoinv/noinv_uni.atm', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s03hjclearinv-ecl-uni/', 
                      's03hjclearinv-ecl-uni-abun.png', 
                      testdirbase+'c03hjclearinv/inv_uni.atm', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass

    try:
        retrievedabun(datadirbase+'s03hjclearinv-tra-uni/', 
                      's03hjclearinv-tra-uni-abun.png', 
                      testdirbase+'c03hjclearinv/inv_uni.atm', 
                      outdir=outdir)
    except Exception as e:
        print(e)
        pass
    """

